// Main program file LAB3.C
// Written by:
// Daniel W. Lewis (dlewis@scu.edu)
// Computer Engineering Department
// Santa Clara University
// Revised Jan 2, 2015
//
// Purpose: Change the brightness and contrast of an image.

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/stat.h>
#include "bmp.h"

IMAGE *		AdjustBrightness(IMAGE *image, int percent) ;
IMAGE *		AdjustContrast(IMAGE *image, int percent) ;

typedef struct
	{
	char	*format ;
	IMAGE	*(*function)() ;
	int		param ;
	} OPTION ;

OPTION *	GetOption(int argc, char **argv, OPTION options[]) ;
void		DisplayImage(char *filespec, char *format, ...) ;

int main(int argc, char **argv)
	{
	char *src_filespec, *dst_filespec ;
	static OPTION options[] =
		{
		{"Brightness (%d)",		AdjustBrightness,	  120},
		{"Contrast (%d)",		AdjustContrast,		  180},
		{NULL,					NULL,				    0}
		} ;
	OPTION *option ;
	IMAGE *image ;

	if (argc != 3 && argc != 4)
		{
		fprintf(stderr, "Usage: %s src-file dst-file {option#}\n", argv[0]) ;
		exit(255) ;
		}

	src_filespec = argv[1] ;
	dst_filespec = argv[2] ;

	image = ReadBMP24(src_filespec) ;

	if ((option = GetOption(argc, argv, options))== NULL)
		{
		fprintf(stderr, "Usage: %s src-file dst-file {option#}\n", argv[0]) ;
		exit(255) ;
		}

	image = (option->function)(image, option->param) ;

	WriteBMP24(dst_filespec, image) ;
	FreeImage(image) ;
	DisplayImage(dst_filespec, option->format, option->param) ;

	return 0 ;
	}

IMAGE *AdjustBrightness(IMAGE *image, int percent)
	{
	unsigned row, rows = GetRows(image) ;
	unsigned col, cols = GetCols(image) ;
	int delta = 255 * (percent - 100) / 100.0 ;

	for (row = 0; row < rows; row++)
		{
		for (col = 0; col < cols; col++)
			{
			RGB rgb = GetRGB(image, row, col) ;
			int level ;

			level = delta + (int) rgb.red ;
			if (level > 255) rgb.red = 255 ;	
			else if (level < 0) rgb.red = 0 ;
			else rgb.red = (uint8_t) level ;

			level = delta + (int) rgb.grn ;
			if (level > 255) rgb.grn = 255 ;	
			else if (level < 0) rgb.grn = 0 ;
			else rgb.grn = (uint8_t) level ;

			level = delta + (int) rgb.blu ;
			if (level > 255) rgb.blu = 255 ;	
			else if (level < 0) rgb.blu = 0 ;
			else rgb.blu = (uint8_t) level ;

			PutRGB(image, row, col, &rgb) ;
			}
		}

	return image ;
	}

IMAGE *AdjustContrast(IMAGE *image, int percent)
	{
	unsigned row, rows = GetRows(image) ;
	unsigned col, cols = GetCols(image) ;
	double factor = percent / 100.0 ;

	for (row = 0; row < rows; row++)
		{
		for (col = 0; col < cols; col++)
			{
			RGB rgb = GetRGB(image, row, col) ;
			int level ;

			level = 128 + factor * ((int) rgb.red - 128) ;
			if (level > 255) level = 255 ;
			else if (level < 0) level = 0 ;
			rgb.red = (unsigned char) level ;

			level = 128 + factor * ((int) rgb.grn - 128) ;
			if (level > 255) level = 255 ;
			else if (level < 0) level = 0 ;
			rgb.grn = (unsigned char) level ;

			level = 128 + factor * ((int) rgb.blu - 128) ;
			if (level > 255) level = 255 ;
			else if (level < 0) level = 0 ;
			rgb.blu = (unsigned char) level ;

			PutRGB(image, row, col, &rgb) ;
			}
		}

	return image ;
	}

OPTION *GetOption(int argc, char **argv, OPTION options[])
	{
	unsigned number, index = 0 ;

	// count number of options
	for (number = 0; options[number].function != NULL; number++) ;

	if (argc == 4)
		{
		sscanf(argv[3], "%u", &index) ;
		}
	else
		{
		printf("\nOptions:\n\n") ;
		for (index = 0; index < number; index++)
			{
			char title[100] ;

			sprintf(title, options[index].format, options[index].param) ;
			printf("%2d: %s\n", index + 1, title) ;
			}

		printf("\nOption? ") ;
		scanf("%u", &index) ;
		}

	return (1 <= index && index <= number) ? &options[index - 1] : NULL ;
	}

void DisplayImage(char *filespec, char *format, ...)
	{
	char progspec[200], command[1000], *program_files, title[100] ;
	struct stat filestat ;
	va_list args ;
	
	va_start(args, format) ;
	vsprintf(title, format, args) ;
	va_end(args) ;

#if defined(_WIN32)
	program_files = getenv("ProgramFiles(x86)") ;
	if (program_files == NULL) program_files = getenv("C:\\Program Files") ;
	if (program_files == NULL) program_files = "" ;

	sprintf(progspec, "%s\\IrfanView\\i_view32.exe", program_files) ;
	if (stat(progspec, &filestat) == 0 && (filestat.st_mode & S_IFREG) != 0)
		{
		sprintf(command, "\"%s\" %s /hide=7 /title=%s", progspec, filespec, title) ;
		}
	else sprintf(command, "mspaint \"%s\"\n", filespec) ;
#elif defined(__unix)
	sprintf(command, "qiv \"%s\"", filespec) ;
#elif defined(__APPLE__)
	sprintf(command, "open \"%s\"", filespec) ;
#endif 
	system(command) ;
	}

